#!/usr/bin/python2

from __future__ import print_function

from oauth import oauth

import argparse
import httplib2
import json
import os
import subprocess
import urllib
import logging

logging.basicConfig(level=logging.INFO)
log = logging.getLogger()
log.name = 'click-sync'

job_template = '%s/job/%s/lastSuccessfulBuild/api/json'
artifacts_template = '%s/job/%s/lastSuccessfulBuild/artifact/%s'

store_uri = 'https://search.apps.ubuntu.com/api/v1/package/%s'

click_packages = [
    ]

click_packages_staged = [
    'camera-app-click',
    ]

store_click_packages = [
    'com.ubuntu.calculator',
    'com.ubuntu.calendar',
    'com.ubuntu.camera',
    'com.ubuntu.clock',
    'com.ubuntu.dropping-letters',
    'com.ubuntu.filemanager',
    'com.ubuntu.gallery',
    'com.ubuntu.notes',
    'com.ubuntu.music',
    'com.ubuntu.stock-ticker-mobile',
    'com.ubuntu.shorts',
    'com.ubuntu.sudoku',
    'com.ubuntu.terminal',
    'com.ubuntu.weather',
    'com.ubuntu.developer.webapps.webapp-amazon',
    'com.ubuntu.developer.webapps.webapp-ebay',
    'com.ubuntu.developer.webapps.webapp-facebook',
    'com.ubuntu.developer.webapps.webapp-gmail',
    'com.ubuntu.developer.webapps.webapp-twitter',
    ]


class OAuthClient(object):

    @classmethod
    def credentials_from_file(cls, credentials_file):
        with open(credentials_file, 'r') as f:
            tokens = json.loads(f.read())
        return cls(**tokens)

    def __init__(self, consumer_key, consumer_secret,
                 token_key, token_secret):
        self.consumer = oauth.OAuthConsumer(key=consumer_key,
                                            secret=consumer_secret)
        self.access_token = oauth.OAuthToken(token_key, token_secret)
        self.signature_method = oauth.OAuthSignatureMethod_PLAINTEXT()

    def get_auth_headers(self, url):
        oauth_request = oauth.OAuthRequest.from_consumer_and_token(
            http_url=url,
            http_method="GET",
            oauth_consumer=self.consumer,
            token=self.access_token
            )
        oauth_request.sign_request(
            self.signature_method, self.consumer, self.access_token)
        return oauth_request.to_header()


def get_json_for_uri(click_uri):
    """
    Returns a json object for the latest successful job from the
    Jenkins uri.
    """
    log.debug(('Getting %s' % click_uri))
    json_request = urllib.urlopen(click_uri)
    if json_request.code != 200:
        raise RuntimeError('%s returned %s with %s' %
                           (click_uri, json_request.code, json_request.read()))
    json_content = json.loads(json_request.read())
    return json_content


def remove_if_previous(click_package, target_dir):
    """
    Locates click_package in target directory and removes it.
    """
    dir_contents = os.listdir(target_dir)
    click_package.split('_')[0]
    matches = filter(lambda x: click_package in x, dir_contents)
    for match in matches:
        target = os.path.join(target_dir, match)
        if os.path.exists(target):
            log.debug('Removing previous click package %s' % match)
            os.unlink(target)


def download(uri, target_file):
    """
    Downloads uri to target_file.
    """
    log.debug(('Downloading %s to %s' % (uri, target_file)))
    new_target_file = "%s.new" % target_file
    try:
        subprocess.check_call(['wget', '--quiet', uri, '-O', new_target_file])
    except subprocess.CalledProcessError:
        if os.path.exists(new_target_file):
            os.unlink(new_target_file)
        raise
    os.rename(new_target_file, target_file)


def get_store_apps(download_dir, auth=None):
    """
    Gets click apps from the store.
    """
    http_client = httplib2.Http()
    download_list = []
    for click in store_click_packages:
        json_content = get_json_for_uri(store_uri % click)
        click_file = json_content['download_url'].split('/')[-1]
        target_file = os.path.join(download_dir, click_file)
        download_url = json_content['download_url']
        if auth:
            auth_headers = auth.get_auth_headers(download_url)
            log.debug('auth_headers: %s' % auth_headers)
        else:
            auth_headers = None
            download_url += '?noauth=1'
            log.warn('Downloading with noauth')
        respr, content = http_client.request(
            download_url, "GET", headers=auth_headers)
        if respr['status'] != '200':
            raise EnvironmentError(respr)
        with open(target_file, 'w') as f:
            f.write(content)
        download_list.append(click_file)
    return download_list


def parse_arguments():
    parser = argparse.ArgumentParser(description='Click package sync tool')
    parser.add_argument('jenkins_url',
                        help='jenkins url to download click packages from.')
    parser.add_argument('download_dir',
                        help='''Path to store the downloaded click packages.
                                The path must exist''')
    parser.add_argument('--credentials-file', default=None, required=False,
                        help='''json dictionary with tokens for oauth.''')
    return parser.parse_args()


if __name__ == '__main__':
    args = parse_arguments()
    download_list = []

    # Jenkins part
    for click_package in click_packages:
        url = job_template % (args.jenkins_url, click_package)
        json_job = get_json_for_uri(url)
        click_package_uri = (artifacts_template %
                             (args.jenkins_url,
                              click_package,
                              json_job['artifacts'][0]['relativePath']))
        click_files = [x['fileName'] for x in json_job['artifacts']
                       if x['fileName'].endswith('.click')]
        for click_file in click_files:
            target_file = os.path.join(args.download_dir, click_file)
            remove_if_previous(click_file, args.download_dir)
            download(click_package_uri, target_file)
            download_list.append(click_file)

    # Store part
    auth = None
    if args.credentials_file:
        auth = OAuthClient.credentials_from_file(args.credentials_file)
    download_list = download_list + get_store_apps(args.download_dir, auth)

    click_list = os.path.join(args.download_dir, 'click_list')
    with open(click_list, 'w') as f:
        for i in download_list:
            f.write('%s\n' % i)
